var project = new Siesta.Project.Browser()

project.configure({
    title               : 'Siesta generic browser examples',
    viewDOM             : true,

    // Define any global JS and CSS dependencies, these files will be injected into each test.
    preload             : [],

    recorderConfig      : {
        // enable/disable certain features of the recorder, see the Siesta.Recorder.Recorder documentation
        recordMouseMoveOnIdle   : true,
        recordPointsOfInterest  : true,
        recordMouseMovePath     : false,
        recordScroll            : false,
        recordInitialWindowSize : false
    }
});

project.plan(
    // Start here, and learn the basic unit testing functionality in Siesta
    {
        group               : 'Unit tests',
        autoCheckGlobals    : true,
        sandbox             : false, // speeds it up, sharing the same iframe for tests in this group

        items : [
            '1.unit-tests/basic_assertions.t.js',
            '1.unit-tests/async_code.t.js',
            project.supportsArrowFunctions ? '1.unit-tests/returning_promise.t.js' : null,
            project.supportsArrowFunctions ? '1.unit-tests/chain_step_with_promise.t.js' : null,
            // sandboxing ecma module test, because of some weird failures in Chrome
            project.supportEcmaModules ? { url: '1.unit-tests/ecma-module.t.js', isEcmaModule: true, sandbox : true } : null,
            '1.unit-tests/todo_tests.t.js',
            project.supportsArrowFunctions ? '1.unit-tests/wait_for.t.js' : null,
            project.supportsArrowFunctions ? {
                // uncomment to "snooze" any failures from this test till 2050 :)
                //snooze          : '2050-01-01',
                url             : '1.unit-tests/bdd.t.js'
            } : null,
            project.supportsArrowFunctions ? '1.unit-tests/spies.t.js' : null,

            // A test descriptor can also be an object with `url` property and its own config options
            {
                // change the title of the corresponding row in the test grid
                title               : 'global variables leakage',

                autoCheckGlobals    : true,
                url                 : '1.unit-tests/global_variables.t.js',

                // change the description (tooltip) of the row
                desc                : 'This test exercises detection of the global namespace pollution'
            },
            // Siesta can also run a Jasmine test suite
            {
                title               : 'Jasmine tests',
                jasmine             : true,
                expectedGlobals     : [ 'Player', 'Song' ],

                // url should point to the specs runner html page in this case
                url                 : '1.unit-tests/jasmine_suite/SpecRunner.html',

                // a link to some external resource, can be activated with CTRL+click on the title
                referenceUrl        : 'https://www.bryntum.com/docs/siesta/#!/api/Siesta.Project.Browser-cfg-jasmine',
                desc                : 'This test shows how existing Jasmine test suite can be run with Siesta'
            }
        ]
    },
    {
        group : 'Web frameworks',
        items : [
            // react example throws exception in IE < 10 - this is not Siesta's fault, so we just hide it in such case
            $.browser.msie && $.browser.version <= 10 ? null :
            {
                group       : 'React',
                pageUrl     : '4.web-frameworks/react/',
                items       : [
                    '4.web-frameworks/react/react.t.js'
                ]
            },
            {
                group       : 'Angular',
                preload     : [
                    '4.web-frameworks/angularjs/angular.min.js',
                    '4.web-frameworks/angularjs/angular-mocks.js',
                    '4.web-frameworks/angularjs/app.js'
                ],
                items       : [
                    '4.web-frameworks/angularjs/angular.t.js'
                ]
            },
            // polymer example throws exception in IE9 - this is not Siesta's fault, so we just hide it in IE9
            bowser.chrome || bowser.safari || bowser.msie && bowser.version <= 9 ? null :
            {
                group       : 'Polymer',
                pageUrl     : '4.web-frameworks/polymer/finished',
                items       : [
                    '4.web-frameworks/polymer/polymer.t.js'
                ]
            },
            {
                group     : 'jQuery',
                testClass : Siesta.Test.jQuery,

                preload : [
                    // Jquery CDN
                    'http://ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js'
                ],

                items : [
                    '4.web-frameworks/jquery/hello_world.t.js',
                    {
                        pageUrl     : '4.web-frameworks/jquery/jquery_ui.html',
                        preload     : 'inherit',
                        alsoPreload : [
                            'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/jquery-ui.min.js',
                            'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/black-tie/jquery-ui.css'
                        ],
                        url         : '4.web-frameworks/jquery/jquery_ui_selectable.t.js'
                    },
                    {
                        url         : '4.web-frameworks/jquery/monkey.t.js'
                    }
                ]
            },
            {
                group     : 'PrototypeJS',
                testClass : Siesta.Test.Browser,
                preload   : [
                    "4.web-frameworks/prototypejs/prototype.js"
                ],
                items     : [
                    '4.web-frameworks/prototypejs/hello_world.t.js'
                ]
            }
        ]
    },
    !project.isStandardPackage() ? null : {
        group               : 'Native events',

        simulation          : 'native',

        items               : [
            {
                pageUrl     : '8.native-events/native.html',
                url         : '8.native-events/010-native.t.js'
            }
        ]
    },
    {
        group   : 'Misc',

        items   : [
            '6.misc/native-dialogs.t.js',
            {
                simulation  : 'synthetic',
                url         : '6.misc/popups.t.js'
            }
        ]
    }
)

project.start()