/*

Siesta 5.1.0
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
!function () {

const child_process     = require('child_process')

const { LogLevel } = require('../src/util/CanLog.js')
const { NodeJsPageRpcFunctionCall } = require('../src/launcher/NodeJsPageRpc')
const { NodeJsPageParentEndPoint } = require('../src/launcher/NodeJsPageRpcParent')


Class('Siesta.Launcher.Page.NodeJS', {

    isa         : Siesta.Launcher.Page.BasePage,

    does        : [
    ],

    has : {
        nodeJsProcess       : null,
        channel             : null
    },


    methods : {

        openHarness : function (projectUrl, callback) {
            // avoid normalization to http url
            return this.SUPER(this.dispatcher.options[ 'project-url' ], callback)
        },


        open : function (url, callback) {
            var me              = this

            var options         = me.dispatcher.options

            // prepare browser args
            var nodeJsArgs      = options[ 'node-arg' ] || []
            if (!(nodeJsArgs instanceof Array)) nodeJsArgs = [ nodeJsArgs ]

            var stdio           = options[ 'isolate-console' ] ? 'ignore' : 'inherit'

            let nodeJsProcess   = me.nodeJsProcess = child_process.fork(
                process.argv[ 1 ],
                [ 'AS_CONNECTOR' ].concat(process.argv.slice(2)),
                {
                    stdio       : [ stdio, stdio, stdio, 'ipc' ],
                    execArgv    : nodeJsArgs//.concat('--inspect-brk=127.0.0.1:9339')
                }
            )

            nodeJsProcess.on('exit', () => {
                me.nodeJsProcess    = null
            })

            me.channel          = new NodeJsPageParentEndPoint({ logLevel : LogLevel.debug })

            me.channel.connect(nodeJsProcess).then(() => callback(), e => callback(e))
        },


        close : function () {
            this.channel.disconnect()

            var nodeJsProcess   = this.nodeJsProcess

            // child process has already exit
            if (!nodeJsProcess) return this.SUPER()

            var me              = this

            nodeJsProcess.kill('SIGTERM')

            return this.SUPER()

            // var promise         = nodeJsProcess.kill('SIGKILL').catch(function (e) {
            //     me.debug("Exception during `nodeJsProcess.kill()`: " + e)
            // })
            //
            // return Promise.all([ promise, this.SUPER() ])
        },


        executeSmallScriptPromised : function (text, ignoreException) {
            var me          = this

            return this.channel.sendRpcCall(
                new NodeJsPageRpcFunctionCall({ func : '(function() {' + text + '})' }).toJson()
            ).then(
                res => res,
                e => {
                    if (!ignoreException) {
                        me.print("<Exception from launcher>")
                        me.print("    While running small script: " + text.substring(0, 150))
                        me.print("    Exception: " + e)
                        me.print("</Exception from launcher>")

                        return Promise.reject(e)
                    } else
                        return Promise.resolve(null)
                }
            )
        }
    }
    // eof methods
})


}();