/*

Siesta 5.1.0
Copyright(c) 2009-2018 Bryntum AB
https://bryntum.com/contact
https://bryntum.com/products/siesta/license

*/
Role('Siesta.Launcher.Role.CanLaunchSimulatorServer', {

    requires : [
        'debug',
        'getOptions'
    ],

    has : {
        simulatorServerClass    : null,

        wsServer                : null
    },


    methods : {

        setupNativeEventsSimulator : function (simServerClass) {
            var options     = this.getOptions()

            var wsServer    = require("../src/simulator/WebSocketConnectionServer")
            var canLog      = require("../src/simulator/util/CanLog")

            var simulatorServerClass = simServerClass || this.simulatorServerClass || require("../src/simulator/robotjs/SimulatorServerRobotJs").SimulatorServerRobotJs

            var server      = this.wsServer = new wsServer.WebSocketConnectionServer({
                simulatorServerClass    : simulatorServerClass,
                logLevel                : options[ 'log-level' ] ? canLog.LogLevel[ options[ 'log-level' ] ] : (options.debug ? canLog.LogLevel.debug : canLog.LogLevel.log)
            })

            var me          = this

            me.debug("Starting native events server")

            return server.start().then(function () {
                me.debug("Native events server has started on port " + server.port + ", awaiting for connections")
            })
        },


        // promised method
        setupNativeEventsSimulatorInSeparateProcess : function (displayNum) {
            var me              = this

            return new Promise(function (resolve, reject) {
                var child_process   = require('child_process');

                me.debug("Starting native events server")

                var childProcess    = child_process.spawn(
                    process.execPath,
                    [ me.launcher.binDir.replace(/\/$/, '') + '/simulator.js', 'auto', JSON.stringify({ displayNumber : displayNum }) ],
                    {}
                )

                var readLineInterface   = require('readline').createInterface({ input : childProcess.stdout })

                readLineInterface.on('line', function (data) {
                    var match   = /port (\d+)/.exec(data)

                    if (match) {
                        me.debug("Native events server has started on port: " + match[ 1 ])

                        me.wsServer.port = Number(match[ 1 ])

                        readLineInterface.close()

                        resolve()
                    }
                });

                childProcess.on('error', reject)
                childProcess.on('exit', reject)

                me.wsServer = {
                    stop : function () {
                        childProcess.kill()

                        var handle  = setTimeout(function () {
                            childProcess.kill('SIGKILL')
                        }, 3000)

                        childProcess.on('exit', function () {
                            clearTimeout(handle)
                        })

                        return Promise.resolve()
                    }
                }
            })
        }
    }
})
