import {Base, Constructable, Mixin} from "../util/Common.js";
import {Channel, Envelop, Message} from "./Types.js";
import {ChannelEndPoint} from "./ChannelEndPoint.js";
import {BrowserLogger} from "../util/browser/BrowserLogger.js";
import {CanSafelyParseJson} from "../util/CanSafelyParseJson.js";


export class BrowserWebSocketChildEndPoint extends ChannelEndPoint(CanSafelyParseJson(BrowserLogger(Base))) {

    socket      : WebSocket

    host        : string
    port        : number


    doConnect (channel : Channel) : Promise<any> {
        return new Promise((resolve, reject) => {
            var socket

            // wrapping WebSocket instantiation in try/catch in attempt to hide the connection exception
            // (if server is not running for example)
            // does not work unfortunately (seems by current browsers design)
            try {
                socket      = this.socket = new WebSocket('ws://' + this.host + ':' + this.port)
            } catch (e) {
            }

            socket.addEventListener('open', event => {
                socket.removeEventListener('close', connectionErrorRejection)
                socket.addEventListener('close', event => {
                    this.onSocketClose(event)

                    this.disconnect()
                })

                socket.addEventListener('message', event => this.onRawChannelMessage(event))

                this.onSocketOpen(event)

                resolve()
            })

            const connectionErrorRejection = event => reject("Connection error")

            socket.addEventListener('close', connectionErrorRejection)
        })
    }


    doDisconnect () {
        const socket        = this.socket

        if (socket && socket.readyState != socket.CLOSED) socket.close()
    }


    onSocketOpen (event : object) {
    }


    onSocketClose (event : object) {
    }


    doSendMessage (message : Message) {
        this.debug("Sending message: " + message)

        this.socket.send(message)
    }


    messageToEnvelop (message : Message) : Envelop | undefined {
        const envelop : Envelop  = this.parseJsonSafely(message) as Envelop

        if (envelop.id == null) return undefined

        return envelop
    }


    envelopToMessage (envelop : Envelop) : Message {
        return JSON.stringify(envelop)
    }
}