import {NodeJsLogger} from "../util/nodejs/NodeJsLogger.js";
import {Base, Constructable} from "../util/Common.js";

const ws                = require('ws')
const portProber        = require('selenium-webdriver/net/portprober')



export class NodeWebSocketConnectionServer extends NodeJsLogger(Base) {
    serverClass             : Constructable
    serverConfig            : any

    port                    : number

    connectionServerWS      : any


    constructor (props : Partial<NodeWebSocketConnectionServer>) {
        super(props)

        Object.assign(this, props)
    }


    start () : Promise<any> {
        const portCont    = this.port ? Promise.resolve(this.port) : portProber.findFreePort()

        return portCont.then(port => {
            return new Promise((resolve, reject) => {

                this.port       = port

                const connectionServerWS    = this.connectionServerWS = new ws.Server({
                    clientTracking      : true,

                    port                : port
                }, resolve);

                connectionServerWS.on('connection', wsClient => {
                    this.debug("Connection received")

                    const cls       = this.serverClass

                    const config    = Object.assign({
                        socket      : wsClient,
                        logLevel    : this.logLevel
                    }, this.serverConfig || {})

                    const webSocketServer = new cls(config)
                });

                connectionServerWS.on('error', () => { this.stop() } );

                // wsServer.on('error', (e) => console.log("ERROR: ", e));
                // wsServer.on('headers', (headers, request) => console.log("HEADERS: ", headers, request));
            })
        })
    }


    stop () {
        return new Promise((resolve, reject) => {
            if (this.connectionServerWS)
                this.connectionServerWS.close(() => {
                    this.connectionServerWS = null

                    resolve()
                })
            else
                resolve()
        })
    }
}